@@ -91,7 +91,6 @@ vAPI.app.restart = function() {
 // chrome.storage.local.get(null, function(bin){ console.debug('%o', bin); });
 
 vAPI.storage = chrome.storage.local;
-vAPI.cacheStorage = chrome.storage.local;
 
 /******************************************************************************/
 /******************************************************************************/
@@ -112,9 +111,7 @@ vAPI.cacheStorage = chrome.storage.local;
 
 vAPI.browserSettings = (function() {
     // Not all platforms support `chrome.privacy`.
-    if ( chrome.privacy instanceof Object === false ) {
-        return;
-    }
+    if ( chrome.privacy instanceof Object === false ) { return; }
 
     return {
         // Whether the WebRTC-related privacy API is crashy is an open question
@@ -123,11 +120,7 @@ vAPI.browserSettings = (function() {
         // an iframe) for platforms where it's a non-issue.
         // https://github.com/uBlockOrigin/uBlock-issues/issues/9
         //   Some Chromium builds are made to look like a Chrome build.
-        webRTCSupported: (function() {
-            if ( vAPI.webextFlavor.soup.has('chromium') === false ) {
-                return true;
-            }
-        })(),
+        webRTCSupported: vAPI.webextFlavor.soup.has('chromium') === false || undefined,
 
         // Calling with `true` means IP address leak is not prevented.
         // https://github.com/gorhill/uBlock/issues/533
@@ -146,16 +139,13 @@ vAPI.browserSettings = (function() {
                 // place.
                 if ( setting ) { return; }
                 this.webRTCSupported = { setting: setting };
-                var iframe = document.createElement('iframe');
-                var me = this;
-                var messageHandler = function(ev) {
-                    if ( ev.origin !== self.location.origin ) {
-                        return;
-                    }
+                let iframe = document.createElement('iframe');
+                const messageHandler = ev => {
+                    if ( ev.origin !== self.location.origin ) { return; }
                     window.removeEventListener('message', messageHandler);
-                    var setting = me.webRTCSupported.setting;
-                    me.webRTCSupported = ev.data === 'webRTCSupported';
-                    me.setWebrtcIPAddress(setting);
+                    const setting = this.webRTCSupported.setting;
+                    this.webRTCSupported = ev.data === 'webRTCSupported';
+                    this.setWebrtcIPAddress(setting);
                     iframe.parentNode.removeChild(iframe);
                     iframe = null;
                 };
@@ -176,12 +166,10 @@ vAPI.browserSettings = (function() {
             // WebRTC not supported: `webRTCMultipleRoutesEnabled` can NOT be
             // safely accessed. Accessing the property will cause full browser
             // crash.
-            if ( this.webRTCSupported !== true ) {
-                return;
-            }
+            if ( this.webRTCSupported !== true ) { return; }
 
-            var cp = chrome.privacy,
-                cpn = cp.network;
+            const cp = chrome.privacy;
+            const cpn = cp.network;
 
             // Older version of Chromium do not support this setting, and is
             // marked as "deprecated" since Chromium 48.
@@ -211,9 +199,14 @@ vAPI.browserSettings = (function() {
                         }, vAPI.resetLastError);
                     } else {
                         // https://github.com/uBlockOrigin/uAssets/issues/333#issuecomment-289426678
-                        // - Leverage virtuous side-effect of strictest setting.
+                        //   Leverage virtuous side-effect of strictest setting.
+                        // https://github.com/gorhill/uBlock/issues/3009
+                        //   Firefox currently works differently, use
+                        //   `default_public_interface_only` for now.
                         cpn.webRTCIPHandlingPolicy.set({
-                            value: 'disable_non_proxied_udp',
+                            value: vAPI.webextFlavor.soup.has('chromium')
+                                ? 'disable_non_proxied_udp'
+                                : 'default_public_interface_only',
                             scope: 'regular'
                         }, vAPI.resetLastError);
                     }
@@ -230,8 +223,9 @@ vAPI.browserSettings = (function() {
                 }
                 switch ( setting ) {
                 case 'prefetching':
+                    const enabled = !!details[setting];
                     try {
-                        if ( !!details[setting] ) {
+                        if ( enabled ) {
                             chrome.privacy.network.networkPredictionEnabled.clear({
                                 scope: 'regular'
                             }, vAPI.resetLastError);
@@ -244,6 +238,9 @@ vAPI.browserSettings = (function() {
                     } catch(ex) {
                         console.error(ex);
                     }
+                    if ( vAPI.prefetching instanceof Function ) {
+                        vAPI.prefetching(enabled);
+                    }
                     break;
 
                 case 'hyperlinkAuditing':
@@ -1038,22 +1035,32 @@ vAPI.messaging.broadcast = function(message) {
 
 // https://github.com/gorhill/uBlock/issues/3474
 // https://github.com/gorhill/uBlock/issues/2823
-// - foil ability of web pages to identify uBO through
+//   Foil ability of web pages to identify uBO through
 //   its web accessible resources.
 // https://github.com/gorhill/uBlock/issues/3497
-// - prevent web pages from interfering with uBO's element picker
+//   Prevent web pages from interfering with uBO's element picker
+// https://github.com/uBlockOrigin/uBlock-issues/issues/550
+//   A specific secret can be used for at most one second.
 
-(function() {
-    vAPI.warSecret =
-        Math.floor(Math.random() * 982451653 + 982451653).toString(36) +
-        Math.floor(Math.random() * 982451653 + 982451653).toString(36);
+vAPI.warSecret = (function() {
+    const generateSecret = ( ) => {
+        return Math.floor(Math.random() * 982451653 + 982451653).toString(36) +
+               Math.floor(Math.random() * 982451653 + 982451653).toString(36);
+    };
 
-    var key = 'secret=' + vAPI.warSecret;
-    var root = vAPI.getURL('/');
-    var guard = function(details) {
-        if ( details.url.indexOf(key) === -1 ) {
+    const root = vAPI.getURL('/');
+    const secrets = [];
+    let lastSecretTime = 0;
+
+    const guard = function(details) {
+        const url = details.url;
+        const pos = secrets.findIndex(secret =>
+            url.lastIndexOf(`?secret=${secret}`) !== -1
+        );
+        if ( pos === -1 ) {
             return { redirectUrl: root };
         }
+        secrets.splice(pos, 1);
     };
 
     chrome.webRequest.onBeforeRequest.addListener(
@@ -1063,6 +1070,20 @@ vAPI.messaging.broadcast = function(message) {
         },
         [ 'blocking' ]
     );
+
+    return ( ) => {
+        if ( secrets.length !== 0 ) {
+            if ( (Date.now() - lastSecretTime) > 5000 ) {
+                secrets.splice(0);
+            } else if ( secrets.length > 256 ) {
+                secrets.splice(0, secrets.length - 192);
+            }
+        }
+        lastSecretTime = Date.now();
+        const secret = generateSecret();
+        secrets.push(secret);
+        return `?secret=${secret}`;
+    };
 })();
 
 vAPI.net = {
