@@ -67,10 +67,10 @@ api.fetchText = function(url, onLoad, onError) {
     let actualUrl = isExternal ? url : vAPI.getURL(url);
 
     // https://github.com/gorhill/uBlock/issues/2592
-    // Force browser cache to be bypassed, but only for resources which have
-    // been fetched more than one hour ago.
+    //   Force browser cache to be bypassed, but only for resources which have
+    //   been fetched more than one hour ago.
     if ( isExternal ) {
-        const queryValue = '_=' + Math.floor(Date.now() / 7200000);
+        const queryValue = `_=${Math.floor(Date.now() / 3600000) % 12}`;
         if ( actualUrl.indexOf('?') === -1 ) {
             actualUrl += '?';
         } else {
@@ -83,6 +83,9 @@ api.fetchText = function(url, onLoad, onError) {
         onError = onLoad;
     }
 
+    return new Promise(resolve => {
+    // Start of executor
+
     const timeoutAfter = µBlock.hiddenSettings.assetFetchTimeout * 1000 || 30000;
     const xhr = new XMLHttpRequest();
     let contentLoaded = 0;
@@ -99,6 +102,20 @@ api.fetchText = function(url, onLoad, onError) {
         }
     };
 
+    const onResolve = function(details) {
+        if ( onLoad instanceof Function ) {
+            return onLoad(details);
+        }
+        resolve(details);
+    };
+
+    const onReject = function(details) {
+        if ( onError instanceof Function ) {
+            return onError(details);
+        }
+        resolve(details);
+    };
+
     // https://github.com/gorhill/uMatrix/issues/15
     const onLoadEvent = function() {
         cleanup();
@@ -110,21 +127,21 @@ api.fetchText = function(url, onLoad, onError) {
             statusText: this.statusText || ''
         };
         if ( details.statusCode < 200 || details.statusCode >= 300 ) {
-            return onError.call(null, details);
+            return onReject(details);
         }
         // consider an empty result to be an error
         if ( stringIsNotEmpty(this.responseText) === false ) {
-            return onError.call(null, details);
+            return onReject(details);
         }
         // we never download anything else than plain text: discard if response
         // appears to be a HTML document: could happen when server serves
         // some kind of error page I suppose
         const text = this.responseText.trim();
         if ( text.startsWith('<') && text.endsWith('>') ) {
-            return onError.call(null, details);
+            return onReject(details);
         }
         details.content = this.responseText;
-        onLoad(details);
+        onResolve(details);
     };
 
     const onErrorEvent = function() {
@@ -134,7 +151,7 @@ api.fetchText = function(url, onLoad, onError) {
             type: 'error',
             text: errorCantConnectTo.replace('{{msg}}', actualUrl)
         });
-        onError({ url, content: '' });
+        onReject({ url, content: '' });
     };
 
     const onTimeout = function() {
@@ -167,6 +184,9 @@ api.fetchText = function(url, onLoad, onError) {
     } catch (e) {
         onErrorEvent.call(xhr);
     }
+
+    // End of executor
+    });
 };
 
 /******************************************************************************/
@@ -396,13 +416,17 @@ const updateAssetSourceRegistry = function(json, silent) {
 
 const getAssetSourceRegistry = function(callback) {
     if ( assetSourceRegistryPromise === undefined ) {
-        assetSourceRegistryPromise = new Promise(resolve => {
-        // start of executor
-        µBlock.cacheStorage.get('assetSourceRegistry', bin => {
+        assetSourceRegistryPromise = µBlock.cacheStorage.get(
+            'assetSourceRegistry'
+        ).then(bin => {
             if (
-                bin instanceof Object === false ||
-                bin.assetSourceRegistry instanceof Object === false
+                bin instanceof Object &&
+                bin.assetSourceRegistry instanceof Object
             ) {
+                assetSourceRegistry = bin.assetSourceRegistry;
+                return;
+            }
+            return new Promise(resolve => {
                 api.fetchText(
                     µBlock.assetsBootstrapLocation || 'assets/assets.json',
                     details => {
@@ -410,12 +434,7 @@ const getAssetSourceRegistry = function(callback) {
                         resolve();
                     }
                 );
-                return;
-            }
-            assetSourceRegistry = bin.assetSourceRegistry;
-            resolve();
-        });
-        // end of executor
+            });
         });
     }
 
@@ -449,26 +468,21 @@ const assetCacheRegistryStartTime = Date.now();
 let assetCacheRegistryPromise;
 let assetCacheRegistry = {};
 
-const getAssetCacheRegistry = function(callback) {
+const getAssetCacheRegistry = function() {
     if ( assetCacheRegistryPromise === undefined ) {
-        assetCacheRegistryPromise = new Promise(resolve => {
-        // start of executor
-        µBlock.cacheStorage.get('assetCacheRegistry', bin => {
+        assetCacheRegistryPromise = µBlock.cacheStorage.get(
+            'assetCacheRegistry'
+        ).then(bin => {
             if (
                 bin instanceof Object &&
                 bin.assetCacheRegistry instanceof Object
             ) {
                 assetCacheRegistry = bin.assetCacheRegistry;
             }
-            resolve();
-        });
-        // end of executor
         });
     }
 
-    assetCacheRegistryPromise.then(( ) => {
-        callback(assetCacheRegistry);
-    });
+    return assetCacheRegistryPromise.then(( ) => assetCacheRegistry);
 };
 
 const saveAssetCacheRegistry = (function() {
@@ -513,11 +527,12 @@ const assetCacheRead = function(assetKey, callback) {
         reportBack(bin[internalKey]);
     };
 
-    let onReady = function() {
-        µBlock.cacheStorage.get(internalKey, onAssetRead);
-    };
-
-    getAssetCacheRegistry(onReady);
+    Promise.all([
+        getAssetCacheRegistry(),
+        µBlock.cacheStorage.get(internalKey),
+    ]).then(results => {
+        onAssetRead(results[1]);
+    });
 };
 
 const assetCacheWrite = function(assetKey, details, callback) {
@@ -550,14 +565,14 @@ const assetCacheWrite = function(assetKey, details, callback) {
         // https://github.com/uBlockOrigin/uBlock-issues/issues/248
         fireNotification('after-asset-updated', result);
     };
-    getAssetCacheRegistry(onReady);
+
+    getAssetCacheRegistry().then(( ) => onReady());
 };
 
 const assetCacheRemove = function(pattern, callback) {
-    const onReady = function() {
-        const cacheDict = assetCacheRegistry,
-            removedEntries = [],
-            removedContent = [];
+    getAssetCacheRegistry().then(cacheDict => {
+        const removedEntries = [];
+        const removedContent = [];
         for ( const assetKey in cacheDict ) {
             if ( pattern instanceof RegExp && !pattern.test(assetKey) ) {
                 continue;
@@ -582,14 +597,15 @@ const assetCacheRemove = function(pattern, callback) {
                 { assetKey: removedEntries[i] }
             );
         }
-    };
-
-    getAssetCacheRegistry(onReady);
+    });
 };
 
 const assetCacheMarkAsDirty = function(pattern, exclude, callback) {
-    const onReady = function() {
-        const cacheDict = assetCacheRegistry;
+    if ( typeof exclude === 'function' ) {
+        callback = exclude;
+        exclude = undefined;
+    }
+    getAssetCacheRegistry().then(cacheDict => {
         let mustSave = false;
         for ( const assetKey in cacheDict ) {
             if ( pattern instanceof RegExp ) {
@@ -617,12 +633,7 @@ const assetCacheMarkAsDirty = function(pattern, exclude, callback) {
         if ( typeof callback === 'function' ) {
             callback();
         }
-    };
-    if ( typeof exclude === 'function' ) {
-        callback = exclude;
-        exclude = undefined;
-    }
-    getAssetCacheRegistry(onReady);
+    });
 };
 
 /******************************************************************************/
@@ -642,63 +653,35 @@ const stringIsNotEmpty = function(s) {
 
 **/
 
-var readUserAsset = function(assetKey, callback) {
-    var reportBack = function(content) {
-        callback({ assetKey: assetKey, content: content });
-    };
+/*******************************************************************************
 
-    var onLoaded = function(bin) {
-        if ( !bin ) { return reportBack(''); }
-        var content = '';
-        if ( typeof bin['cached_asset_content://assets/user/filters.txt'] === 'string' ) {
-            content = bin['cached_asset_content://assets/user/filters.txt'];
-            µBlock.cacheStorage.remove('cached_asset_content://assets/user/filters.txt');
-        }
-        if ( typeof bin['assets/user/filters.txt'] === 'string' ) {
-            content = bin['assets/user/filters.txt'];
-            // TODO(seamless migration):
-            // Uncomment once all moved to v1.11+.
-            //vAPI.storage.remove('assets/user/filters.txt');
-        }
-        if ( typeof bin[assetKey] === 'string' ) {
-            // TODO(seamless migration):
-            // Replace conditional with assignment once all moved to v1.11+
-            if ( content !== bin[assetKey] ) {
-                saveUserAsset(assetKey, content);
-            }
-        } else if ( content !== '' ) {
-            saveUserAsset(assetKey, content);
-        }
-        return reportBack(content);
+    User assets are NOT persisted in the cache storage. User assets are
+    recognized by the asset key which always starts with 'user-'.
+
+**/
+
+const readUserAsset = function(assetKey, callback) {
+    const reportBack = function(content) {
+        callback({ assetKey, content });
     };
-    var toRead = assetKey;
-    if ( assetKey === µBlock.userFiltersPath ) {
-        toRead = [
-            assetKey,
-            'assets/user/filters.txt',
-            'cached_asset_content://assets/user/filters.txt'
-        ];
-    }
-    vAPI.storage.get(toRead, onLoaded);
+    vAPI.storage.get(assetKey, bin => {
+        const content =
+            bin instanceof Object && typeof bin[assetKey] === 'string'
+                ? bin[assetKey]
+                : '';
+        return reportBack(content);
+    });
+    // Remove obsolete entry
+    // TODO: remove once everybody is well beyond 1.18.6
+    vAPI.storage.remove('assets/user/filters.txt');
 };
 
-var saveUserAsset = function(assetKey, content, callback) {
-    var bin = {};
-    bin[assetKey] = content;
-    // TODO(seamless migration):
-    // This is for forward compatibility. Only for a limited time. Remove when
-    // everybody moved to 1.11.0 and beyond.
-    // >>>>>>>>
-    if ( assetKey === µBlock.userFiltersPath ) {
-        bin['assets/user/filters.txt'] = content;
-    }
-    // <<<<<<<<
-    var onSaved = function() {
+const saveUserAsset = function(assetKey, content, callback) {
+    vAPI.storage.set({ [assetKey]: content }, ( ) => {
         if ( callback instanceof Function ) {
-            callback({ assetKey: assetKey, content: content });
+            callback({ assetKey, content });
         }
-    };
-    vAPI.storage.set(bin, onSaved);
+    });
 };
 
 /******************************************************************************/
@@ -711,27 +694,33 @@ api.get = function(assetKey, options, callback) {
         callback = noopfunc;
     }
 
+    return new Promise(resolve => {
+    // start of executor
     if ( assetKey === µBlock.userFiltersPath ) {
-        readUserAsset(assetKey, callback);
+        readUserAsset(assetKey, details => {
+            callback(details);
+            resolve(details);
+        });
         return;
     }
 
-    var assetDetails = {},
+    let assetDetails = {},
         contentURLs,
         contentURL;
 
-    var reportBack = function(content, err) {
-        var details = { assetKey: assetKey, content: content };
+    const reportBack = function(content, err) {
+        const details = { assetKey: assetKey, content: content };
         if ( err ) {
             details.error = assetDetails.lastError = err;
         } else {
             assetDetails.lastError = undefined;
         }
         callback(details);
+        resolve(details);
     };
 
-    var onContentNotLoaded = function() {
-        var isExternal;
+    const onContentNotLoaded = function() {
+        let isExternal;
         while ( (contentURL = contentURLs.shift()) ) {
             isExternal = reIsExternalPath.test(contentURL);
             if ( isExternal === false || assetDetails.hasLocalURL !== true ) {
@@ -748,7 +737,7 @@ api.get = function(assetKey, options, callback) {
         }
     };
 
-    var onContentLoaded = function(details) {
+    const onContentLoaded = function(details) {
         if ( stringIsNotEmpty(details.content) === false ) {
             onContentNotLoaded();
             return;
@@ -762,7 +751,7 @@ api.get = function(assetKey, options, callback) {
         reportBack(details.content);
     };
 
-    var onCachedContentLoaded = function(details) {
+    const onCachedContentLoaded = function(details) {
         if ( details.content !== '' ) {
             return reportBack(details.content);
         }
@@ -780,11 +769,13 @@ api.get = function(assetKey, options, callback) {
     };
 
     assetCacheRead(assetKey, onCachedContentLoaded);
+    // end of executor
+    });
 };
 
 /******************************************************************************/
 
-var getRemote = function(assetKey, callback) {
+const getRemote = function(assetKey, callback) {
    var assetDetails = {},
         contentURLs,
         contentURL;
@@ -852,10 +843,19 @@ var getRemote = function(assetKey, callback) {
 /******************************************************************************/
 
 api.put = function(assetKey, content, callback) {
-    if ( reIsUserAsset.test(assetKey) ) {
-        return saveUserAsset(assetKey, content, callback);
-    }
-    assetCacheWrite(assetKey, content, callback);
+    return new Promise(resolve => {
+        const onDone = function(details) {
+            if ( typeof callback === 'function' ) {
+                callback(details);
+            }
+            resolve(details);
+        };
+        if ( reIsUserAsset.test(assetKey) ) {
+            saveUserAsset(assetKey, content, onDone);
+        } else {
+            assetCacheWrite(assetKey, content, onDone);
+        }
+    });
 };
 
 /******************************************************************************/
@@ -895,7 +895,7 @@ api.metadata = function(callback) {
         if ( cacheRegistryReady ) { onReady(); }
     });
 
-    getAssetCacheRegistry(function() {
+    getAssetCacheRegistry().then(( ) => {
         cacheRegistryReady = true;
         if ( assetRegistryReady ) { onReady(); }
     });
@@ -1013,7 +1013,7 @@ var updateNext = function() {
         updateOne();
     });
 
-    getAssetCacheRegistry(function(dict) {
+    getAssetCacheRegistry().then(dict => {
         cacheDict = dict;
         if ( !assetDict ) { return; }
         updateOne();
