@@ -35,9 +35,6 @@ if ( vAPI.webextFlavor === undefined ) {
 
 const µBlock = (function() { // jshint ignore:line
 
-    const oneSecond = 1000,
-          oneMinute = 60 * oneSecond;
-
     const hiddenSettingsDefault = {
         assetFetchTimeout: 30,
         autoCommentFilterTemplate: '{{date}} {{origin}}',
@@ -46,6 +43,7 @@ const µBlock = (function() { // jshint ignore:line
         cacheStorageAPI: 'unset',
         cacheStorageCompression: true,
         cacheControlForFirefox1376932: 'no-cache, no-store, must-revalidate',
+        consoleLogLevel: 'unset',
         debugScriptlets: false,
         disableWebAssembly: false,
         ignoreRedirectFilters: false,
@@ -53,8 +51,9 @@ const µBlock = (function() { // jshint ignore:line
         manualUpdateAssetFetchPeriod: 500,
         popupFontSize: 'unset',
         requestJournalProcessPeriod: 1000,
+        selfieAfter: 11,
         strictBlockingBypassDuration: 120,
-        suspendTabsUntilReady: false,
+        suspendTabsUntilReady: 'unset',
         userResourcesLocation: 'unset'
     };
 
@@ -95,24 +94,25 @@ const µBlock = (function() { // jshint ignore:line
 
         hiddenSettingsDefault: hiddenSettingsDefault,
         hiddenSettings: (function() {
-            let out = Object.assign({}, hiddenSettingsDefault),
-                json = vAPI.localStorage.getItem('immediateHiddenSettings');
-            if ( typeof json === 'string' ) {
-                try {
-                    let o = JSON.parse(json);
-                    if ( o instanceof Object ) {
-                        for ( let k in o ) {
-                            if ( out.hasOwnProperty(k) ) {
-                                out[k] = o[k];
-                            }
-                        }
+            const out = Object.assign({}, hiddenSettingsDefault);
+            const json = vAPI.localStorage.getItem('immediateHiddenSettings');
+            if ( typeof json !== 'string' ) { return out; }
+            try {
+                const o = JSON.parse(json);
+                if ( o instanceof Object ) {
+                    for ( const k in o ) {
+                        if ( out.hasOwnProperty(k) ) { out[k] = o[k]; }
+                    }
+                    self.log.verbosity = out.consoleLogLevel;
+                    if ( typeof out.suspendTabsUntilReady === 'boolean' ) {
+                        out.suspendTabsUntilReady = out.suspendTabsUntilReady
+                            ? 'yes'
+                            : 'unset';
                     }
                 }
-                catch(ex) {
-                }
             }
-            // Remove once 1.15.12+ is widespread.
-            vAPI.localStorage.removeItem('hiddenSettings');
+            catch(ex) {
+            }
             return out;
         })(),
 
@@ -137,8 +137,8 @@ const µBlock = (function() { // jshint ignore:line
 
         // Read-only
         systemSettings: {
-            compiledMagic: 6,   // Increase when compiled format changes
-            selfieMagic: 7      // Increase when selfie format changes
+            compiledMagic: 14,  // Increase when compiled format changes
+            selfieMagic: 14     // Increase when selfie format changes
         },
 
         restoreBackupSettings: {
@@ -161,8 +161,6 @@ const µBlock = (function() { // jshint ignore:line
         selectedFilterLists: [],
         availableFilterLists: {},
 
-        selfieAfter: 17 * oneMinute,
-
         pageStores: new Map(),
         pageStoresToken: 0,
 
