@@ -144,14 +144,6 @@
         let dbPromise;
         let dbTimer;
 
-        const genericErrorHandler = function(ev) {
-            const error = ev.target && ev.target.error;
-            if ( error && error.name === 'QuotaExceededError' ) {
-                api.error = error.name;
-            }
-            console.error('[%s]', STORAGE_NAME, error && error.name);
-        };
-
         const noopfn = function () {
         };
 
@@ -216,20 +208,18 @@
                     return resolve(null);
                 }
                 req.onupgradeneeded = function(ev) {
-                    req = undefined;
-                    const db = ev.target.result;
-                    db.onerror = db.onabort = genericErrorHandler;
-                    const table = db.createObjectStore(
-                        STORAGE_NAME,
-                        { keyPath: 'key' }
-                    );
-                    table.createIndex('value', 'value', { unique: false });
+                    if ( ev.oldVersion === 1 ) { return; }
+                    try {
+                        const db = ev.target.result;
+                        db.createObjectStore(STORAGE_NAME, { keyPath: 'key' });
+                    } catch(ex) {
+                        req.onerror();
+                    }
                 };
                 req.onsuccess = function(ev) {
                     if ( resolve === undefined ) { return; }
                     req = undefined;
                     db = ev.target.result;
-                    db.onerror = db.onabort = genericErrorHandler;
                     dbPromise = undefined;
                     resolve(db);
                     resolve = undefined;
@@ -296,7 +286,7 @@
         const visitAllFromDb = function(visitFn) {
             getDb().then(db => {
                 if ( !db ) { return visitFn(); }
-                const transaction = db.transaction(STORAGE_NAME);
+                const transaction = db.transaction(STORAGE_NAME, 'readonly');
                 transaction.oncomplete =
                 transaction.onerror =
                 transaction.onabort = ( ) => visitFn();
@@ -333,6 +323,9 @@
                         keyvalStore[result.key] = result.value;
                     })
                 );
+            }).catch(reason => {
+                console.info(`cacheStorage.getAllFromDb() failed: ${reason}`);
+                callback();
             });
         };
 
@@ -352,22 +345,13 @@
             const entries = [];
             const dontCompress =
                 µBlock.hiddenSettings.cacheStorageCompression !== true;
-            let bytesInUse = 0;
             const handleEncodingResult = result => {
-                if ( typeof result.data === 'string' ) {
-                    bytesInUse += result.data.length;
-                } else if ( result.data instanceof Blob ) {
-                    bytesInUse += result.data.size;
-                }
                 entries.push({ key: result.key, value: result.data });
             };
             for ( const key of keys ) {
                 const data = keyvalStore[key];
                 const isString = typeof data === 'string';
                 if ( isString === false || dontCompress ) {
-                    if ( isString ) {
-                        bytesInUse += data.length;
-                    }
                     entries.push({ key, value: data });
                     continue;
                 }
@@ -382,7 +366,7 @@
                     if ( callback === undefined ) { return; }
                     let cb = callback;
                     callback = undefined;
-                    cb({ bytesInUse });
+                    cb();
                 };
                 try {
                     const transaction = db.transaction(
@@ -435,15 +419,18 @@
             if ( typeof callback !== 'function' ) {
                 callback = noopfn;
             }
-            disconnect();
-            try {
-                const req = indexedDB.deleteDatabase(STORAGE_NAME);
-                req.onsuccess = req.onerror = ( ) => {
+            getDb().then(db => {
+                let transaction = db.transaction(STORAGE_NAME, 'readwrite');
+                transaction.oncomplete =
+                transaction.onerror =
+                transaction.onabort = ( ) => {
                     callback();
                 };
-            } catch(ex) {
+                transaction.objectStore(STORAGE_NAME).clear();
+            }).catch(reason => {
+                console.info(`cacheStorage.clearDb() failed: ${reason}`);
                 callback();
-            }
+            });
         };
 
         return getDb().then(db => {
