@@ -170,7 +170,7 @@
         return cm.getSearchCursor(
             query,
             pos,
-            {caseFold: queryCaseInsensitive(query), multiline: true}
+            { caseFold: queryCaseInsensitive(query), multiline: false }
         );
     }
 
@@ -209,7 +209,8 @@
             }
             state.annotate = cm.showMatchesOnScrollbar(
                 state.query,
-                queryCaseInsensitive(state.query)
+                queryCaseInsensitive(state.query),
+                { multiline: false }
             );
             let count = state.annotate.matches.length;
             state.widget
