@@ -123,43 +123,42 @@ const HNTRIE_CHAR0_SLOT  = HNTRIE_TRIE0_SLOT + 2;         //  66 / 264
 const HNTRIE_CHAR1_SLOT  = HNTRIE_TRIE0_SLOT + 3;         //  67 / 268
 const HNTRIE_TRIE0_START = HNTRIE_TRIE0_SLOT + 4 << 2;    //       272
 
+const HNTrieContainer = class {
 
-const HNTrieContainer = function(details) {
-    if ( details instanceof Object === false ) { details = {}; }
-    let len = (details.byteLength || 0) + HNTRIE_PAGE_SIZE-1 & ~(HNTRIE_PAGE_SIZE-1);
-    this.buf = new Uint8Array(Math.max(len, 131072));
-    this.buf32 = new Uint32Array(this.buf.buffer);
-    this.needle = '';
-    this.buf32[HNTRIE_TRIE0_SLOT] = HNTRIE_TRIE0_START;
-    this.buf32[HNTRIE_TRIE1_SLOT] = this.buf32[HNTRIE_TRIE0_SLOT];
-    this.buf32[HNTRIE_CHAR0_SLOT] = details.char0 || 65536;
-    this.buf32[HNTRIE_CHAR1_SLOT] = this.buf32[HNTRIE_CHAR0_SLOT];
-    this.wasmInstancePromise = null;
-    this.wasmMemory = null;
-    this.readyToUse();
-};
-
-HNTrieContainer.prototype = {
+    constructor(details) {
+        if ( details instanceof Object === false ) { details = {}; }
+        let len = (details.byteLength || 0) + HNTRIE_PAGE_SIZE-1 & ~(HNTRIE_PAGE_SIZE-1);
+        this.buf = new Uint8Array(Math.max(len, 131072));
+        this.buf32 = new Uint32Array(this.buf.buffer);
+        this.needle = '';
+        this.buf32[HNTRIE_TRIE0_SLOT] = HNTRIE_TRIE0_START;
+        this.buf32[HNTRIE_TRIE1_SLOT] = this.buf32[HNTRIE_TRIE0_SLOT];
+        this.buf32[HNTRIE_CHAR0_SLOT] = details.char0 || 65536;
+        this.buf32[HNTRIE_CHAR1_SLOT] = this.buf32[HNTRIE_CHAR0_SLOT];
+        this.wasmInstancePromise = null;
+        this.wasmMemory = null;
+        this.readyToUse();
+    }
 
     //--------------------------------------------------------------------------
     // Public methods
     //--------------------------------------------------------------------------
 
-    reset: function() {
+    reset() {
         this.buf32[HNTRIE_TRIE1_SLOT] = this.buf32[HNTRIE_TRIE0_SLOT];
         this.buf32[HNTRIE_CHAR1_SLOT] = this.buf32[HNTRIE_CHAR0_SLOT];
-    },
+    }
 
-    readyToUse: function() {
+    readyToUse() {
         if ( HNTrieContainer.wasmModulePromise instanceof Promise === false ) {
             return Promise.resolve();
         }
         return HNTrieContainer.wasmModulePromise.then(
             module => this.initWASM(module)
         );
-    },
+    }
 
-    setNeedle: function(needle) {
+    setNeedle(needle) {
         if ( needle !== this.needle ) {
             const buf = this.buf;
             let i = needle.length;
@@ -171,12 +170,13 @@ HNTrieContainer.prototype = {
             this.needle = needle;
         }
         return this;
-    },
+    }
 
-    matchesJS: function(iroot) {
+    matchesJS(iroot) {
         const char0 = this.buf32[HNTRIE_CHAR0_SLOT];
         let ineedle = this.buf[255];
-        let icell = iroot;
+        let icell = this.buf32[iroot+0];
+        if ( icell === 0 ) { return -1; }
         for (;;) {
             if ( ineedle === 0 ) { return -1; }
             ineedle -= 1;
@@ -214,11 +214,9 @@ HNTrieContainer.prototype = {
             }
         }
         return ineedle === 0 || this.buf[ineedle-1] === 0x2E ? ineedle : -1;
-    },
-    matchesWASM: null,
-    matches: null,
+    }
 
-    createOne: function(args) {
+    createOne(args) {
         if ( Array.isArray(args) ) {
             return new this.HNTrieRef(this, args[0], args[1]);
         }
@@ -232,21 +230,15 @@ HNTrieContainer.prototype = {
         this.buf32[iroot+1] = 0;
         this.buf32[iroot+2] = 0;
         return new this.HNTrieRef(this, iroot, 0);
-    },
+    }
 
-    compileOne: function(trieRef) {
+    compileOne(trieRef) {
         return [ trieRef.iroot, trieRef.size ];
-    },
+    }
 
-    addJS: function(iroot) {
+    addJS(iroot) {
         let lhnchar = this.buf[255];
         if ( lhnchar === 0 ) { return 0; }
-        let icell = iroot;
-        // special case: first node in trie
-        if ( this.buf32[icell+2] === 0 ) {
-            this.buf32[icell+2] = this.addSegment(lhnchar);
-            return 1;
-        }
         // grow buffer if needed
         if (
             (this.buf32[HNTRIE_CHAR0_SLOT] - this.buf32[HNTRIE_TRIE1_SLOT]) < 24 ||
@@ -254,6 +246,12 @@ HNTrieContainer.prototype = {
         ) {
             this.growBuf(24, 256);
         }
+        let icell = this.buf32[iroot+0];
+        // special case: first node in trie
+        if ( icell === 0 ) {
+            this.buf32[iroot+0] = this.addCell(0, 0, this.addSegment(lhnchar));
+            return 1;
+        }
         //
         const char0 = this.buf32[HNTRIE_CHAR0_SLOT];
         let inext;
@@ -262,6 +260,9 @@ HNTrieContainer.prototype = {
             const vseg = this.buf32[icell+2];
             // skip boundary cells
             if ( vseg === 0 ) {
+                // remainder is at label boundary? if yes, no need to add
+                // the rest since the shortest match is always reported
+                if ( this.buf[lhnchar-1] === 0x2E /* '.' */ ) { return -1; }
                 icell = this.buf32[icell+1];
                 continue;
             }
@@ -306,6 +307,9 @@ HNTrieContainer.prototype = {
                         icell = inext;
                         continue;
                     }
+                    // remainder is at label boundary? if yes, no need to add
+                    // the rest since the shortest match is always reported
+                    if ( this.buf[lhnchar-1] === 0x2E /* '.' */ ) { return -1; }
                     // boundary cell + needle remainder
                     inext = this.addCell(0, 0, 0);
                     this.buf32[icell+1] = inext;
@@ -334,28 +338,26 @@ HNTrieContainer.prototype = {
             }
             return 1;
         }
-    },
-    addWASM: null,
-    add: null,
+    }
 
-    optimize: function() {
+    optimize() {
         this.shrinkBuf();
         return {
             byteLength: this.buf.byteLength,
             char0: this.buf32[HNTRIE_CHAR0_SLOT],
         };
-    },
+    }
 
-    fromIterable: function(hostnames, add) {
+    fromIterable(hostnames, add) {
         if ( add === undefined ) { add = 'add'; }
         const trieRef = this.createOne();
         for ( const hn of hostnames ) {
             trieRef[add](hn);
         }
         return trieRef;
-    },
+    }
 
-    serialize: function(encoder) {
+    serialize(encoder) {
         if ( encoder instanceof Object ) {
             return encoder.encode(
                 this.buf32.buffer,
@@ -369,16 +371,16 @@ HNTrieContainer.prototype = {
                 this.buf32[HNTRIE_CHAR1_SLOT] + 3 >>> 2
             )
         );
-    },
+    }
 
-    unserialize: function(selfie, decoder) {
+    unserialize(selfie, decoder) {
         this.needle = '';
         const shouldDecode = typeof selfie === 'string';
         let byteLength = shouldDecode
             ? decoder.decodeSize(selfie)
             : selfie.length << 2;
+        if ( byteLength === 0 ) { return false; }
         byteLength = byteLength + HNTRIE_PAGE_SIZE-1 & ~(HNTRIE_PAGE_SIZE-1);
-        if ( byteLength === 0 ) { return; }
         if ( this.wasmMemory !== null ) {
             const pageCountBefore = this.buf.length >>> 16;
             const pageCountAfter = byteLength >>> 16;
@@ -396,23 +398,14 @@ HNTrieContainer.prototype = {
         } else {
             this.buf32.set(selfie);
         }
-    },
-
-    //--------------------------------------------------------------------------
-    // Class to hold reference to a specific trie
-    //--------------------------------------------------------------------------
-
-    HNTrieRef: function(container, iroot, size) {
-        this.container = container;
-        this.iroot = iroot;
-        this.size = size;
-    },
+        return true;
+    }
 
     //--------------------------------------------------------------------------
     // Private methods
     //--------------------------------------------------------------------------
 
-    addCell: function(idown, iright, v) {
+    addCell(idown, iright, v) {
         let icell = this.buf32[HNTRIE_TRIE1_SLOT];
         this.buf32[HNTRIE_TRIE1_SLOT] = icell + 12;
         icell >>>= 2;
@@ -420,9 +413,9 @@ HNTrieContainer.prototype = {
         this.buf32[icell+1] = iright;
         this.buf32[icell+2] = v;
         return icell;
-    },
+    }
 
-    addSegment: function(lsegchar) {
+    addSegment(lsegchar) {
         if ( lsegchar === 0 ) { return 0; }
         let char1 = this.buf32[HNTRIE_CHAR1_SLOT];
         const isegchar = char1 - this.buf32[HNTRIE_CHAR0_SLOT];
@@ -432,9 +425,9 @@ HNTrieContainer.prototype = {
         } while ( i !== 0 );
         this.buf32[HNTRIE_CHAR1_SLOT] = char1;
         return (lsegchar << 24) | isegchar;
-    },
+    }
 
-    growBuf: function(trieGrow, charGrow) {
+    growBuf(trieGrow, charGrow) {
         const char0 = Math.max(
             (this.buf32[HNTRIE_TRIE1_SLOT] + trieGrow + HNTRIE_PAGE_SIZE-1) & ~(HNTRIE_PAGE_SIZE-1),
             this.buf32[HNTRIE_CHAR0_SLOT]
@@ -445,18 +438,18 @@ HNTrieContainer.prototype = {
             this.buf.length
         );
         this.resizeBuf(bufLen, char0);
-    },
+    }
 
-    shrinkBuf: function() {
+    shrinkBuf() {
         // Can't shrink WebAssembly.Memory
         if ( this.wasmMemory !== null ) { return; }
         const char0 = this.buf32[HNTRIE_TRIE1_SLOT] + 24;
         const char1 = char0 + this.buf32[HNTRIE_CHAR1_SLOT] - this.buf32[HNTRIE_CHAR0_SLOT];
         const bufLen = char1 + 256;
         this.resizeBuf(bufLen, char0);
-    },
+    }
 
-    resizeBuf: function(bufLen, char0) {
+    resizeBuf(bufLen, char0) {
         bufLen = bufLen + HNTRIE_PAGE_SIZE-1 & ~(HNTRIE_PAGE_SIZE-1);
         if (
             bufLen === this.buf.length &&
@@ -507,9 +500,9 @@ HNTrieContainer.prototype = {
             this.buf32[HNTRIE_CHAR0_SLOT] = char0;
             this.buf32[HNTRIE_CHAR1_SLOT] = char0 + charDataLen;
         }
-    },
+    }
 
-    initWASM: function(module) {
+    initWASM(module) {
         if ( module instanceof WebAssembly.Module === false ) {
             return Promise.resolve(null);
         }
@@ -526,9 +519,10 @@ HNTrieContainer.prototype = {
             );
             this.wasmInstancePromise.then(instance => {
                 this.wasmMemory = memory;
-                const pageCount = this.buf.byteLength + HNTRIE_PAGE_SIZE-1 >>> 16;
-                if ( pageCount > 1 ) {
-                    memory.grow(pageCount - 1);
+                const curPageCount = memory.buffer.byteLength >>> 16;
+                const newPageCount = this.buf.byteLength + HNTRIE_PAGE_SIZE-1 >>> 16;
+                if ( newPageCount > curPageCount ) {
+                    memory.grow(newPageCount - curPageCount);
                 }
                 const buf = new Uint8Array(memory.buffer);
                 buf.set(this.buf);
@@ -539,43 +533,98 @@ HNTrieContainer.prototype = {
             });
         }
         return this.wasmInstancePromise;
-    },
+    }
 };
 
-/******************************************************************************/
+HNTrieContainer.prototype.matches = HNTrieContainer.prototype.matchesJS;
+HNTrieContainer.prototype.matchesWASM = null;
 
-HNTrieContainer.prototype.HNTrieRef.prototype = {
-    add: function(hn) {
-        if ( this.container.setNeedle(hn).add(this.iroot) === 1 ) {
+HNTrieContainer.prototype.add = HNTrieContainer.prototype.addJS;
+HNTrieContainer.prototype.addWASM = null;
+
+/*******************************************************************************
+
+    Class to hold reference to a specific trie
+
+*/
+
+HNTrieContainer.prototype.HNTrieRef = class {
+
+    constructor(container, iroot, size) {
+        this.container = container;
+        this.iroot = iroot;
+        this.size = size;
+    }
+
+    add(hn) {
+        if ( this.container.setNeedle(hn).add(this.iroot) > 0 ) {
+            this.last = -1;
+            this.needle = '';
             this.size += 1;
             return true;
         }
         return false;
-    },
-    addJS: function(hn) {
-        if ( this.container.setNeedle(hn).addJS(this.iroot) === 1 ) {
+    }
+
+    addJS(hn) {
+        if ( this.container.setNeedle(hn).addJS(this.iroot) > 0 ) {
+            this.last = -1;
+            this.needle = '';
             this.size += 1;
             return true;
         }
         return false;
-    },
-    addWASM: function(hn) {
-        if ( this.container.setNeedle(hn).addWASM(this.iroot) === 1 ) {
+    }
+
+    addWASM(hn) {
+        if ( this.container.setNeedle(hn).addWASM(this.iroot) > 0 ) {
+            this.last = -1;
+            this.needle = '';
             this.size += 1;
             return true;
         }
         return false;
-    },
-    matches: function(needle) {
-        return this.container.setNeedle(needle).matches(this.iroot);
-    },
-    matchesJS: function(needle) {
-        return this.container.setNeedle(needle).matchesJS(this.iroot);
-    },
-    matchesWASM: function(needle) {
-        return this.container.setNeedle(needle).matchesWASM(this.iroot);
-    },
-    [Symbol.iterator]: function() {
+    }
+
+    matches(needle) {
+        if ( needle !== this.needle ) {
+            this.needle = needle;
+            this.last = this.container.setNeedle(needle).matches(this.iroot);
+        }
+        return this.last;
+    }
+
+    matchesJS(needle) {
+        if ( needle !== this.needle ) {
+            this.needle = needle;
+            this.last = this.container.setNeedle(needle).matchesJS(this.iroot);
+        }
+        return this.last;
+    }
+
+    matchesWASM(needle) {
+        if ( needle !== this.needle ) {
+            this.needle = needle;
+            this.last = this.container.setNeedle(needle).matchesWASM(this.iroot);
+        }
+        return this.last;
+    }
+
+    dump() {
+        let hostnames = Array.from(this);
+        if ( String.prototype.padStart instanceof Function ) {
+            const maxlen = Math.min(
+                hostnames.reduce((maxlen, hn) => Math.max(maxlen, hn.length), 0),
+                64
+            );
+            hostnames = hostnames.map(hn => hn.padStart(maxlen));
+        }
+        for ( const hn of hostnames ) {
+            console.log(hn);
+        }
+    }
+
+    [Symbol.iterator]() {
         return {
             value: undefined,
             done: false,
@@ -625,9 +674,12 @@ HNTrieContainer.prototype.HNTrieRef.prototype = {
             forks: [],
             textDecoder: new TextDecoder()
         };
-    },
+    }
 };
 
+HNTrieContainer.prototype.HNTrieRef.prototype.last = -1;
+HNTrieContainer.prototype.HNTrieRef.prototype.needle = '';
+
 /******************************************************************************/
 
 // Code below is to attempt to load a WASM module which implements:
@@ -641,10 +693,6 @@ HNTrieContainer.prototype.HNTrieRef.prototype = {
 (function() {
     HNTrieContainer.wasmModulePromise = null;
 
-    // Default to javascript version.
-    HNTrieContainer.prototype.matches = HNTrieContainer.prototype.matchesJS;
-    HNTrieContainer.prototype.add = HNTrieContainer.prototype.addJS;
-
     if (
         typeof WebAssembly !== 'object' ||
         typeof WebAssembly.compileStreaming !== 'function'
