@@ -167,20 +167,24 @@
                 'matches-css-after',
                 'matches-css-before',
                 'not',
+                'nth-ancestor',
                 'watch-attrs',
                 'xpath'
                 ].join('|'),
             ')\\('
         ].join(''));
 
-        const reEscapeRegex = /[.*+?^${}()|[\]\\]/g,
-            reNeedScope = /^\s*[+>~]/,
-            reIsDanglingSelector = /(?:[+>~]\s*|\s+)$/;
+        const reEatBackslashes = /\\([()])/g;
+        const reEscapeRegex = /[.*+?^${}()|[\]\\]/g;
+        const reNeedScope = /^\s*[+>~]/;
+        const reIsDanglingSelector = /(?:[+>~]\s*|\s+)$/;
 
         const regexToRawValue = new Map();
         let lastProceduralSelector = '',
             lastProceduralSelectorCompiled;
 
+        // When dealing with literal text, we must first eat _some_
+        // backslash characters.
         const compileText = function(s) {
             const match = reParseRegexLiteral.exec(s);
             let regexDetails;
@@ -191,7 +195,8 @@
                     regexDetails = [ regexDetails, match[2] ];
                 }
             } else {
-                regexDetails = s.replace(reEscapeRegex, '\\$&');
+                regexDetails = s.replace(reEatBackslashes, '$1')
+                                .replace(reEscapeRegex, '\\$&');
                 regexToRawValue.set(regexDetails, s);
             }
             return regexDetails;
@@ -236,6 +241,13 @@
             }
         };
 
+        const compileNthAncestorSelector = function(s) {
+            const n = parseInt(s, 10);
+            if ( isNaN(n) === false && n >= 1 && n < 256 ) {
+                return n;
+            }
+        };
+
         const compileSpathExpression = function(s) {
             if ( isValidCSSSelector('*' + s) ) {
                 return s;
@@ -278,6 +290,7 @@
             [ ':matches-css-after', compileCSSDeclaration ],
             [ ':matches-css-before', compileCSSDeclaration ],
             [ ':not', compileNotSelector ],
+            [ ':nth-ancestor', compileNthAncestorSelector ],
             [ ':spath', compileSpathExpression ],
             [ ':watch-attrs', compileAttrList ],
             [ ':xpath', compileXpathExpression ]
@@ -301,42 +314,45 @@
                 switch ( task[0] ) {
                 case ':has':
                 case ':if':
-                    raw.push(':has', '(', decompile(task[1]), ')');
+                    raw.push(`:has(${decompile(task[1])})`);
                     break;
                 case ':has-text':
                     if ( Array.isArray(task[1]) ) {
-                        value = '/' + task[1][0] + '/' + task[1][1];
+                        value = `/${task[1][0]}/${task[1][1]}`;
                     } else {
                         value = regexToRawValue.get(task[1]);
                         if ( value === undefined ) {
-                            value = '/' + task[1] + '/';
+                            value = `/${task[1]}/`;
                         }
                     }
-                    raw.push(task[0], '(', value, ')');
+                    raw.push(`:has-text(${value})`);
                     break;
                 case ':matches-css':
                 case ':matches-css-after':
                 case ':matches-css-before':
                     if ( Array.isArray(task[1].value) ) {
-                        value = '/' + task[1].value[0] + '/' + task[1].value[1];
+                        value = `/${task[1].value[0]}/${task[1].value[1]}`;
                     } else {
                         value = regexToRawValue.get(task[1].value);
                         if ( value === undefined ) {
-                            value = '/' + task[1].value + '/';
+                            value = `/${task[1].value}/`;
                         }
                     }
-                    raw.push(task[0], '(', task[1].name, ': ', value, ')');
+                    raw.push(`${task[0]}(${task[1].name}: ${value})`);
                     break;
                 case ':not':
                 case ':if-not':
-                    raw.push(':not', '(', decompile(task[1]), ')');
+                    raw.push(`:not(${decompile(task[1])})`);
+                    break;
+                case ':nth-ancestor':
+                    raw.push(`:nth-ancestor(${task[1]})`);
                     break;
                 case ':spath':
                     raw.push(task[1]);
                     break;
                 case ':watch-attrs':
                 case ':xpath':
-                    raw.push(task[0], '(', task[1], ')');
+                    raw.push(`${task[0]}(${task[1]})`);
                     break;
                 }
             }
@@ -463,7 +479,7 @@
             return µb.cosmeticFilteringEngine.discardedCount +
                    µb.scriptletFilteringEngine.discardedCount +
                    µb.htmlFilteringEngine.discardedCount;
-        }
+        },
     };
 
     //--------------------------------------------------------------------------
@@ -660,6 +676,8 @@
         };
 
         const entryPoint = function(raw) {
+            entryPoint.pseudoclass = false;
+
             const extendedSyntax = reExtendedSyntax.test(raw);
             if ( isValidCSSSelector(raw) && extendedSyntax === false ) {
                 return raw;
@@ -668,18 +686,18 @@
             // We  rarely reach this point -- majority of selectors are plain
             // CSS selectors.
 
-            let matches, operator;
+            let matches;
 
-            // Supported Adguard/ABP advanced selector syntax: will translate into
-            // uBO's syntax before further processing.
+            // Supported Adguard/ABP advanced selector syntax: will translate
+            // into uBO's syntax before further processing.
             // Mind unsupported advanced selector syntax, such as ABP's
             // `-abp-properties`.
-            // Note: extended selector syntax has been deprecated in ABP, in favor
-            // of the procedural one (i.e. `:operator(...)`). See
-            // https://issues.adblockplus.org/ticket/5287
+            // Note: extended selector syntax has been deprecated in ABP, in
+            // favor of the procedural one (i.e. `:operator(...)`).
+            // See https://issues.adblockplus.org/ticket/5287
             if ( extendedSyntax ) {
                 while ( (matches = reExtendedSyntaxParser.exec(raw)) !== null ) {
-                    operator = normalizedExtendedSyntaxOperators.get(matches[1]);
+                    const operator = normalizedExtendedSyntaxOperators.get(matches[1]);
                     if ( operator === undefined ) { return; }
                     raw = raw.slice(0, matches.index) +
                           operator + '(' + matches[3] + ')' +
@@ -688,8 +706,7 @@
                 return entryPoint(raw);
             }
 
-            let selector = raw,
-                pseudoclass, style;
+            let selector = raw, pseudoclass, style;
 
             // `:style` selector?
             if ( (matches = reStyleSelector.exec(selector)) !== null ) {
@@ -705,23 +722,16 @@
             }
 
             if ( style !== undefined || pseudoclass !== undefined ) {
-                if ( isValidCSSSelector(selector) === false ) {
-                    return;
-                }
+                if ( isValidCSSSelector(selector) === false ) { return; }
                 if ( pseudoclass !== undefined ) {
                     selector += pseudoclass;
                 }
                 if ( style !== undefined ) {
                     if ( isValidStyleProperty(style) === false ) { return; }
-                    return JSON.stringify({
-                        raw: raw,
-                        style: [ selector, style ]
-                    });
+                    return JSON.stringify({ raw, style: [ selector, style ] });
                 }
-                return JSON.stringify({
-                    raw: raw,
-                    pseudoclass: true
-                });
+                entryPoint.pseudoclass = true;
+                return JSON.stringify({ raw, pseudoclass: true });
             }
 
             // Procedural selector?
@@ -731,6 +741,8 @@
             }
         };
 
+        entryPoint.pseudoclass = false;
+
         return entryPoint;
     })();
 
