@@ -50,10 +50,13 @@
         countdown += 1;
         vAPI.storage.getBytesInUse(null, process);
     }
-    if ( this.cacheStorage.name !== 'browser.storage.local' ) {
+    if (
+        navigator.storage instanceof Object &&
+        navigator.storage.estimate instanceof Function
+    ) {
         countdown += 1;
-        this.assets.getBytesInUse().then(count => {
-            process(count);
+        navigator.storage.estimate().then(estimate => {
+            process(estimate.usage);
         });
     }
     if ( countdown === 0 ) {
@@ -985,41 +988,37 @@
 
 /******************************************************************************/
 
-µBlock.loadRedirectResources = function(updatedContent) {
-    let content = '';
+µBlock.loadRedirectResources = function() {
+    return this.redirectEngine.resourcesFromSelfie().then(success => {
+        if ( success === true ) { return; }
+
+        const fetchPromises = [ this.assets.get('ublock-resources') ];
+
+        const userResourcesLocation = this.hiddenSettings.userResourcesLocation;
+        if ( userResourcesLocation !== 'unset' ) {
+            for ( const url of userResourcesLocation.split(/\s+/) ) {
+                fetchPromises.push(this.assets.fetchText(url));
+            }
+        }
+
+        return Promise.all(fetchPromises);
+    }).then(results => {
+        if ( Array.isArray(results) === false ) { return; }
+
+        let content = '';
+
+        for ( const result of results ) {
+            if (
+                result instanceof Object === false ||
+                typeof result.content !== 'string' ||
+                result.content === ''
+            ) {
+                continue;
+            }
+            content += '\n\n' + result.content;
+        }
 
-    const onDone = ( ) => {
         this.redirectEngine.resourcesFromString(content);
-    };
-
-    const onUserResourcesLoaded = details => {
-        if ( details.content !== '' ) {
-            content += '\n\n' + details.content;
-        }
-        onDone();
-    };
-
-    const onResourcesLoaded = details => {
-        if ( details.content !== '' ) {
-            content = details.content;
-        }
-        if ( this.hiddenSettings.userResourcesLocation === 'unset' ) {
-            return onDone();
-        }
-        this.assets.fetchText(
-            this.hiddenSettings.userResourcesLocation,
-            onUserResourcesLoaded
-        );
-    };
-
-    if ( typeof updatedContent === 'string' && updatedContent.length !== 0 ) {
-        return onResourcesLoaded({ content: updatedContent });
-    }
-
-    this.redirectEngine.resourcesFromSelfie().then(success => {
-        if ( success !== true ) {
-            this.assets.get('ublock-resources', onResourcesLoaded);
-        }
     });
 };
 
@@ -1033,7 +1032,7 @@
     return this.assets.get(
         'compiled/' + this.pslAssetKey
     ).then(details =>
-        publicSuffixList.fromSelfie(details.content, µBlock.base128)
+        publicSuffixList.fromSelfie(details.content, µBlock.base64)
     ).catch(reason => {
         console.info(reason);
         return false;
@@ -1051,7 +1050,7 @@
     publicSuffixList.parse(content, punycode.toASCII);
     this.assets.put(
         'compiled/' + this.pslAssetKey,
-        publicSuffixList.toSelfie(µBlock.base128)
+        publicSuffixList.toSelfie(µBlock.base64)
     );
 };
 
@@ -1245,19 +1244,21 @@
 //   Support ability to auto-enable a filter list based on user agent.
 
 µBlock.listMatchesEnvironment = function(details) {
-    var re;
     // Matches language?
     if ( typeof details.lang === 'string' ) {
-        re = this.listMatchesEnvironment.reLang;
+        let re = this.listMatchesEnvironment.reLang;
         if ( re === undefined ) {
-            re = new RegExp('\\b' + self.navigator.language.slice(0, 2) + '\\b');
-            this.listMatchesEnvironment.reLang = re;
+            const match = /^[a-z]+/.exec(self.navigator.language);
+            if ( match !== null ) {
+                re = new RegExp('\\b' + match[0] + '\\b');
+                this.listMatchesEnvironment.reLang = re;
+            }
         }
-        if ( re.test(details.lang) ) { return true; }
+        if ( re !== undefined && re.test(details.lang) ) { return true; }
     }
     // Matches user agent?
     if ( typeof details.ua === 'string' ) {
-        re = new RegExp('\\b' + this.escapeRegex(details.ua) + '\\b', 'i');
+        let re = new RegExp('\\b' + this.escapeRegex(details.ua) + '\\b', 'i');
         if ( re.test(self.navigator.userAgent) ) { return true; }
     }
     return false;
@@ -1348,9 +1349,6 @@
             }
         } else if ( details.assetKey === 'ublock-resources' ) {
             this.redirectEngine.invalidateResourcesSelfie();
-            if ( cached ) {
-                this.loadRedirectResources(details.content);
-            }
         }
         vAPI.messaging.broadcast({
             what: 'assetUpdated',
