@@ -41,57 +41,122 @@
 // Benchmark for string-based tokens vs. safe-integer token values:
 //   https://gorhill.github.io/obj-vs-set-vs-map/tokenize-to-str-vs-to-int.html
 
-µBlock.urlTokenizer = {
-    setURL: function(url) {
+µBlock.urlTokenizer = new (class {
+    constructor() {
+        this._chars = '0123456789%abcdefghijklmnopqrstuvwxyz';
+        this._validTokenChars = new Uint8Array(128);
+        for ( let i = 0, n = this._chars.length; i < n; i++ ) {
+            this._validTokenChars[this._chars.charCodeAt(i)] = i + 1;
+        }
+
+        // Four upper bits of token hash are reserved for built-in predefined
+        // token hashes, which should never end up being used when tokenizing
+        // any arbitrary string.
+             this.dotTokenHash = 0x10000000;
+             this.anyTokenHash = 0x20000000;
+        this.anyHTTPSTokenHash = 0x30000000;
+         this.anyHTTPTokenHash = 0x40000000;
+              this.noTokenHash = 0x50000000;
+           this.emptyTokenHash = 0xF0000000;
+
+        this._urlIn = '';
+        this._urlOut = '';
+        this._tokenized = false;
+        this._tokens = [ 0 ];
+
+        this.knownTokens = new Uint8Array(65536);
+        this.resetKnownTokens();
+    }
+
+    setURL(url) {
         if ( url !== this._urlIn ) {
             this._urlIn = url;
             this._urlOut = url.toLowerCase();
             this._tokenized = false;
         }
         return this._urlOut;
-    },
+    }
+
+    resetKnownTokens() {
+        this.knownTokens.fill(0);
+        this.addKnownToken(this.dotTokenHash);
+        this.addKnownToken(this.anyTokenHash);
+        this.addKnownToken(this.anyHTTPSTokenHash);
+        this.addKnownToken(this.anyHTTPTokenHash);
+        this.addKnownToken(this.noTokenHash);
+    }
+
+    addKnownToken(th) {
+        this.knownTokens[th & 0xFFFF ^ th >>> 16] = 1;
+    }
 
     // Tokenize on demand.
-    getTokens: function() {
-        if ( this._tokenized === false ) {
-            this._tokenize();
-            this._tokenized = true;
+    getTokens() {
+        if ( this._tokenized ) { return this._tokens; }
+        let i = this._tokenize();
+        i = this._appendTokenAt(i, this.anyTokenHash, 0);
+        if ( this._urlOut.startsWith('https://') ) {
+            i = this._appendTokenAt(i, this.anyHTTPSTokenHash, 0);
+        } else if ( this._urlOut.startsWith('http://') ) {
+            i = this._appendTokenAt(i, this.anyHTTPTokenHash, 0);
         }
+        i = this._appendTokenAt(i, this.noTokenHash, 0);
+        this._tokens[i] = 0;
+        this._tokenized = true;
         return this._tokens;
-    },
+    }
 
-    tokenHashFromString: function(s) {
-        var l = s.length;
-        if ( l === 0 ) { return 0; }
-        if ( l === 1 ) {
-            if ( s === '*' ) { return 63; }
-            if ( s === '.' ) { return 62; }
-        }
-        var vtc = this._validTokenChars,
-            th = vtc[s.charCodeAt(0)];
-        for ( var i = 1; i !== 8 && i !== l; i++ ) {
-            th = th * 64 + vtc[s.charCodeAt(i)];
+    tokenHashFromString(s) {
+        const l = s.length;
+        if ( l === 0 ) { return this.emptyTokenHash; }
+        const vtc = this._validTokenChars;
+        let th = vtc[s.charCodeAt(0)];
+        for ( let i = 1; i !== 7 && i !== l; i++ ) {
+            th = th << 4 ^ vtc[s.charCodeAt(i)];
         }
         return th;
-    },
+    }
+
+    stringFromTokenHash(th) {
+        if ( th === 0 ) { return ''; }
+        return th.toString(16);
+    }
+
+    toSelfie() {
+        return µBlock.base64.encode(
+            this.knownTokens.buffer,
+            this.knownTokens.byteLength
+        );
+    }
+
+    fromSelfie(selfie) {
+        return µBlock.base64.decode(selfie, this.knownTokens.buffer);
+    }
 
     // https://github.com/chrisaljoudi/uBlock/issues/1118
     // We limit to a maximum number of tokens.
 
-    _tokenize: function() {
-        var tokens = this._tokens,
-            url = this._urlOut,
-            l = url.length;
-        if ( l === 0 ) { tokens[0] = 0; return; }
+    _appendTokenAt(i, th, ti) {
+        this._tokens[i+0] = th;
+        this._tokens[i+1] = ti;
+        return i + 2;
+    }
+
+    _tokenize() {
+        const tokens = this._tokens;
+        let url = this._urlOut;
+        let l = url.length;
+        if ( l === 0 ) { return this.emptyTokenHash; }
         if ( l > 2048 ) {
             url = url.slice(0, 2048);
             l = 2048;
         }
-        var i = 0, j = 0, v, n, ti, th,
-            vtc = this._validTokenChars;
+        const knownTokens = this.knownTokens;
+        const vtc = this._validTokenChars;
+        let i = 0, j = 0, v, n, ti, th;
         for (;;) {
             for (;;) {
-                if ( i === l ) { tokens[j] = 0; return; }
+                if ( i === l ) { return j; }
                 v = vtc[url.charCodeAt(i++)];
                 if ( v !== 0 ) { break; }
             }
@@ -100,29 +165,18 @@
                 if ( i === l ) { break; }
                 v = vtc[url.charCodeAt(i++)];
                 if ( v === 0 ) { break; }
-                if ( n === 8 ) { continue; }
-                th = th * 64 + v;
+                if ( n === 7 ) { continue; }
+                th = th << 4 ^ v;
                 n += 1;
             }
-            tokens[j++] = th;
-            tokens[j++] = ti;
+            if ( knownTokens[th & 0xFFFF ^ th >>> 16] !== 0 ) {
+                tokens[j+0] = th;
+                tokens[j+1] = ti;
+                j += 2;
+            }
         }
-    },
-
-    _urlIn: '',
-    _urlOut: '',
-    _tokenized: false,
-    _tokens: [ 0 ],
-    _validTokenChars: (function() {
-        var vtc = new Uint8Array(128),
-            chars = '0123456789%abcdefghijklmnopqrstuvwxyz',
-            i = chars.length;
-        while ( i-- ) {
-            vtc[chars.charCodeAt(i)] = i + 1;
-        }
-        return vtc;
-    })()
-};
+    }
+})();
 
 /******************************************************************************/
 
@@ -499,7 +553,7 @@
 
 /******************************************************************************/
 
-// Custom base128 encoder/decoder
+// Custom base64 encoder/decoder
 //
 // TODO:
 //   Could expand the LZ4 codec API to be able to return UTF8-safe string
@@ -510,163 +564,99 @@
 //   JSON string. The fallback can be removed once min supported version is
 //   above 59.
 
-µBlock.base128 = {
-    encode: function(arrbuf, arrlen) {
-        if (
-            vAPI.webextFlavor.soup.has('chromium') &&
-            vAPI.webextFlavor.major < 60
-        ) {
-            return this.encodeJSON(arrbuf);
-        }
-        return this.encodeBase128(arrbuf, arrlen);
-    },
-    encodeBase128: function(arrbuf, arrlen) {
-        const inbuf = new Uint8Array(arrbuf, 0, arrlen);
-        const inputLength = arrlen;
-        let _7cnt = Math.floor(inputLength / 7);
-        let outputLength = _7cnt * 8;
-        let _7rem = inputLength % 7;
-        if ( _7rem !== 0 ) {
-            outputLength += 1 + _7rem;
+µBlock.base64 = new (class {
+    constructor() {
+        this.valToDigit = new Uint8Array(64);
+        this.digitToVal = new Uint8Array(128);
+        const chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz@%";
+        for ( let i = 0, n = chars.length; i < n; i++ ) {
+            const c = chars.charCodeAt(i);
+            this.valToDigit[i] = c;
+            this.digitToVal[c] = i;
         }
+        this.magic = 'Base64_1';
+    }
+
+    encode(arrbuf, arrlen) {
+        const inputLength = arrlen >>> 2;
+        const inbuf = new Uint32Array(arrbuf, 0, inputLength);
+        const outputLength = this.magic.length + 7 + inputLength * 7;
         const outbuf = new Uint8Array(outputLength);
-        let msbits, v;
-        let i = 0, j = 0;
-        while ( _7cnt--  ) {
-            v = inbuf[i+0];
-            msbits  = (v & 0x80) >>> 7;
-            outbuf[j+1] = v & 0x7F;
-            v = inbuf[i+1];
-            msbits |= (v & 0x80) >>> 6;
-            outbuf[j+2] = v & 0x7F;
-            v = inbuf[i+2];
-            msbits |= (v & 0x80) >>> 5;
-            outbuf[j+3] = v & 0x7F;
-            v = inbuf[i+3];
-            msbits |= (v & 0x80) >>> 4;
-            outbuf[j+4] = v & 0x7F;
-            v = inbuf[i+4];
-            msbits |= (v & 0x80) >>> 3;
-            outbuf[j+5] = v & 0x7F;
-            v = inbuf[i+5];
-            msbits |= (v & 0x80) >>> 2;
-            outbuf[j+6] = v & 0x7F;
-            v = inbuf[i+6];
-            msbits |= (v & 0x80) >>> 1;
-            outbuf[j+7] = v & 0x7F;
-            outbuf[j+0] = msbits;
-            i += 7; j += 8;
+        let j = 0;
+        for ( let i = 0; i < this.magic.length; i++ ) {
+            outbuf[j++] = this.magic.charCodeAt(i);
         }
-        if ( _7rem > 0 ) {
-            msbits = 0;
-            for ( let ir = 0; ir < _7rem; ir++ ) {
-                v = inbuf[i+ir];
-                msbits |= (v & 0x80) >>> (7 - ir);
-                outbuf[j+ir+1] = v & 0x7F;
-            }
-            outbuf[j+0] = msbits;
+        let v = inputLength;
+        do {
+            outbuf[j++] = this.valToDigit[v & 0b111111];
+            v >>>= 6;
+        } while ( v !== 0 );
+        outbuf[j++] = 0x20 /* ' ' */;
+        for ( let i = 0; i < inputLength; i++ ) {
+            v = inbuf[i];
+            do {
+                outbuf[j++] = this.valToDigit[v & 0b111111];
+                v >>>= 6;
+            } while ( v !== 0 );
+            outbuf[j++] = 0x20 /* ' ' */;
+        }
+        if ( typeof TextDecoder === 'undefined' ) {
+            return JSON.stringify(
+                Array.from(new Uint32Array(outbuf.buffer, 0, j >>> 2))
+            );
         }
         const textDecoder = new TextDecoder();
-        return textDecoder.decode(outbuf);
-    },
-    encodeJSON: function(arrbuf) {
-        return JSON.stringify(Array.from(new Uint32Array(arrbuf)));
-    },
-    // TODO:
-    //   Surprisingly, there does not seem to be any performance gain when
-    //   first converting the input string into a Uint8Array through
-    //   TextEncoder. Investigate again to confirm original findings and
-    //   to find out whether results have changed. Not using TextEncoder()
-    //   to create an intermediate input buffer lower peak memory usage
-    //   at selfie load time.
-    //
-    //   const textEncoder = new TextEncoder();
-    //   const inbuf = textEncoder.encode(instr);
-    //   const inputLength = inbuf.byteLength;
-    decode: function(instr, arrbuf) {
-        if ( instr.length === 0 ) { return; }
-        if ( instr.charCodeAt(0) === 0x5B /* '[' */ ) {
-            const outbuf = this.decodeJSON(instr, arrbuf);
-            if ( outbuf !== undefined ) {
-                return outbuf;
+        return textDecoder.decode(new Uint8Array(outbuf.buffer, 0, j));
+    }
+
+    decode(instr, arrbuf) {
+        if (  instr.charCodeAt(0) === 0x5B /* '[' */ ) {
+            const inbuf = JSON.parse(instr);
+            if ( arrbuf instanceof ArrayBuffer === false ) {
+                return new Uint32Array(inbuf);
             }
+            const outbuf = new Uint32Array(arrbuf);
+            outbuf.set(inbuf);
+            return outbuf;
         }
-        if (
-            vAPI.webextFlavor.soup.has('chromium') &&
-            vAPI.webextFlavor.major < 60
-        ) {
-            throw new Error('Unexpected µBlock.base128 encoding');
+        if ( instr.startsWith(this.magic) === false ) {
+            throw new Error('Invalid µBlock.base64 encoding');
         }
-        return this.decodeBase128(instr, arrbuf);
-    },
-    decodeBase128: function(instr, arrbuf) {
         const inputLength = instr.length;
-        let _8cnt = inputLength >>> 3;
-        let outputLength = _8cnt * 7;
-        let _8rem = inputLength % 8;
-        if ( _8rem !== 0 ) {
-            outputLength += _8rem - 1;
-        }
         const outbuf = arrbuf instanceof ArrayBuffer === false
-            ? new Uint8Array(outputLength)
-            : new Uint8Array(arrbuf);
-        let msbits;
-        let i = 0, j = 0;
-        while ( _8cnt-- ) {
-            msbits = instr.charCodeAt(i+0);
-            outbuf[j+0] = msbits << 7 & 0x80 | instr.charCodeAt(i+1);
-            outbuf[j+1] = msbits << 6 & 0x80 | instr.charCodeAt(i+2);
-            outbuf[j+2] = msbits << 5 & 0x80 | instr.charCodeAt(i+3);
-            outbuf[j+3] = msbits << 4 & 0x80 | instr.charCodeAt(i+4);
-            outbuf[j+4] = msbits << 3 & 0x80 | instr.charCodeAt(i+5);
-            outbuf[j+5] = msbits << 2 & 0x80 | instr.charCodeAt(i+6);
-            outbuf[j+6] = msbits << 1 & 0x80 | instr.charCodeAt(i+7);
-            i += 8; j += 7;
+            ? new Uint32Array(this.decodeSize(instr))
+            : new Uint32Array(arrbuf);
+        let i = instr.indexOf(' ', this.magic.length) + 1;
+        if ( i === -1 ) {
+            throw new Error('Invalid µBlock.base64 encoding');
         }
-        if ( _8rem > 1 ) {
-            msbits = instr.charCodeAt(i+0);
-            for ( let ir = 1; ir < _8rem; ir++ ) {
-                outbuf[j+ir-1] = msbits << (8-ir) & 0x80 | instr.charCodeAt(i+ir);
+        let j = 0;
+        for (;;) {
+            if ( i === inputLength ) { break; }
+            let v = 0, l = 0;
+            for (;;) {
+                const c = instr.charCodeAt(i++);
+                if ( c === 0x20 /* ' ' */ ) { break; }
+                v += this.digitToVal[c] << l;
+                l += 6;
             }
+            outbuf[j++] = v;
         }
         return outbuf;
-    },
-    decodeJSON: function(instr, arrbuf) {
-        let buf;
-        try {
-            buf = JSON.parse(instr);
-        } catch (ex) {
+    }
+
+    decodeSize(instr) {
+        if ( instr.startsWith(this.magic) === false ) { return 0; }
+        let v = 0, l = 0, i = this.magic.length;
+        for (;;) {
+            const c = instr.charCodeAt(i++);
+            if ( c === 0x20 /* ' ' */ ) { break; }
+            v += this.digitToVal[c] << l;
+            l += 6;
         }
-        if ( Array.isArray(buf) === false ) { return; }
-        const outbuf = arrbuf instanceof ArrayBuffer === false
-            ? new Uint32Array(buf.length << 2)
-            : new Uint32Array(arrbuf);
-        outbuf.set(buf);
-        return new Uint8Array(outbuf.buffer);
-    },
-    decodeSize: function(instr) {
-        if ( instr.length === 0 ) { return 0; }
-        if ( instr.charCodeAt(0) === 0x5B /* '[' */ ) {
-            let buf;
-            try {
-                buf = JSON.parse(instr);
-            } catch (ex) {
-            }
-            if ( Array.isArray(buf) ) {
-                return buf.length << 2;
-            }
-        }
-        if (
-            vAPI.webextFlavor.soup.has('chromium') &&
-            vAPI.webextFlavor.major < 60
-        ) {
-            throw new Error('Unexpected µBlock.base128 encoding');
-        }
-        const size = (instr.length >>> 3) * 7;
-        const rem = instr.length & 7;
-        return rem === 0 ? size : size + rem - 1;
-    },
-};
+        return v << 2;
+    }
+})();
 
 /******************************************************************************/
 
@@ -719,31 +709,26 @@
             return datasetPromise;
         }
 
-        datasetPromise = new Promise(resolve => {
-            console.info(`Loading benchmark dataset...`);
-            const url = vAPI.getURL('/assets/requests.json');
-            µBlock.assets.fetchText(url, details => {
-                if ( details.error !== undefined ) {
-                    datasetPromise = undefined;
-                    console.info(`Not found: ${url}`);
-                    resolve();
-                    return;
+        console.info(`Loading benchmark dataset...`);
+        const url = vAPI.getURL('/assets/requests.json');
+        datasetPromise = µBlock.assets.fetchText(url).then(details => {
+            console.info(`Parsing benchmark dataset...`);
+            const requests = [];
+            const lineIter = new µBlock.LineIterator(details.content);
+            while ( lineIter.eot() === false ) {
+                let request;
+                try {
+                    request = JSON.parse(lineIter.next());
+                } catch(ex) {
                 }
-                console.info(`Parsing benchmark dataset...`);
-                const requests = [];
-                const lineIter = new µBlock.LineIterator(details.content);
-                while ( lineIter.eot() === false ) {
-                    let request;
-                    try {
-                        request = JSON.parse(lineIter.next());
-                    } catch(ex) {
-                    }
-                    if ( request instanceof Object === false ) { continue; }
-                    if ( !request.frameUrl || !request.url ) { continue; }
-                    requests.push(request);
-                }
-                resolve(requests);
-            });
+                if ( request instanceof Object === false ) { continue; }
+                if ( !request.frameUrl || !request.url ) { continue; }
+                requests.push(request);
+            }
+            return requests;
+        }).catch(details => {
+            console.info(`Not found: ${details.url}`);
+            datasetPromise = undefined;
         });
 
         return datasetPromise;
